///////////////////////////////////////////////////////////////////////////////////////////////////////////////
//=============================================================================================================
// INTRODUCTION
//=============================================================================================================
//  Project/Example: Sound Sensitive LED (X02)
//  Purpose:  A simple project that uses the Microphone to detect sound levels and maps it to the the
//              brightness of the LED Matrix.
//  License: GNU General Public License v3
//  Link: https://wiki.microduino.cc/index.php/Sound_Sensitive_LED_Project_(X02)
// 
//  Author(s): Microduino, Sonny Sung
//  E-mail(s): support at microduino.cc, sonny at microduino.cc
//
//  Version: 1.0
//  Date: 2017/04/20
//  Notes: Initial version
//=============================================================================================================
// REQUIRED HARDWARE
//=============================================================================================================
//  COMPATIBLE KITS
//    [ ] 101 Basic Kit
//    [ ] 201 Advanced Kit
//    [ ] 301 Expert Kit
//    [*] 102 Basic Kit
//    [*] 202 Advanced Kit
//    [*] 302 Expert Kit
//    [ ] Itty Bitty City Kit
//=============================================================================================================
//  MODULES
//    [1] Core, Core+, CoreUSB -OR- CoreRF
//    [1] mBattery -OR- USBTTL
//    [1] Sensor Hub
//    [1] LED Matrix
//  SENSORS/TRINKETS
//    [2] Sensor Cable
//    [1] Microphone Sensor
//    [1] Touch Button
//  OTHER
//    [1] MicroUSB Cable
//=============================================================================================================
// SETUP
//=============================================================================================================
//  NOTE: When connecting sensor wires, push on the plastic connector and not on the wires. 
//          Pushing on the wire can damage them.
//
//  1. Connect the Microphone Sensor to the Sensor Hub on Pin A2/A3
//  2. Connect the Touch Button to the Sensor Hub on Pin 6/7
//  3. Grab the mCookie USBTTL or mBattery and Stack the mCookie Core on top.
//  4. Stack the mCookie LED Matrix on top of that.
//  5. Stack the mCookie Sensor on top of that.
//  6. Plug in the MicroUSB cable to the mCookie USBTTL or mBattery to a computer.
//  7. Program the example by pressing "upload".
//=============================================================================================================
// USAGE
//=============================================================================================================
//  Talk or play music towards the Microphone and the LED Matrix should react to the noise loudness. Change 
//  the color of the LED Matrix by pressing the Touch Button. There 7 select-able colors. There is a 200 
//  millisecond (1/5 second) delay between Touch Button detection.
//=============================================================================================================
// TWEAKING
//=============================================================================================================
//  "userDef.h" tab has configurable variables.
//  Visit https://wiki.microduino.cc/index.php/Sound_Sensitive_LED_Project_(X02)#Tweaking for further tweaking 
//    instructions.
//=============================================================================================================
// NOTES
//=============================================================================================================
//  *More info at: https://wiki.microduino.cc/index.php/Sound_Sensitive_LED_Project_(X02)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "userDef.h"
#include <Microduino_ColorLED.h>    //Import the library for the ColorLED.
#include <stdint.h>

unsigned long debounce_time_touch_pin = 0;

ColorLED matrix = ColorLED(NUMPIXELS, LEDMATRIX_PIN, NEO_GRB + NEO_KHZ800);

uint8_t current_selection = 0;
const uint8_t CURRENT_SELECTION_COUNT = 7;

uint8_t color_matrix[CURRENT_SELECTION_COUNT][3]  = {
  {1, 0, 0},
  {0, 1, 0},
  {0, 0, 1},
  {0, 1, 1},
  {1, 0, 1},
  {1, 1, 0},
  {1, 1, 1}
};


void setup() {
  Serial.begin(9600);
  // put your setup code here, to run once:
  pinMode(MIC_PIN, INPUT);
  pinMode(TOUCH_PIN, INPUT);

  matrix.begin();
  matrix.show();
}

void loop() {
  // put your main code here, to run repeatedly:

  //Check if touch button is pressed.
  if ( !digitalRead(TOUCH_PIN )
       &&
       ((millis() - debounce_time_touch_pin ) > DEBOUNCE_TIME_MILLISECONDS)
     ) {
    debounce_time_touch_pin = millis();

    //Change the current selection color by incrementing the value
    if (++current_selection >= CURRENT_SELECTION_COUNT) {
      current_selection = 0;
    }
  }

  //Read in the microphone value
  uint16_t mic_value = analogRead(MIC_PIN);

#ifdef DEBUG
  Serial.println(mic_value);
  delay(1);
#endif

  //Check if the microphone value is under the ambient threshold, set mic value to 0 to ignore if it is under the threshold
  if (mic_value < AMBIENT_THRESHOLD) {
    mic_value = 0;
  }
  //Check if the microphone value is over the max loudness threshold
  //If it is, set the microphone value to that threshold, this sets the max value for the microphone value
  if (mic_value > MAX_THRESHOLD) {
    mic_value = MAX_THRESHOLD;
  }

  //Determine the brightness of the leds by using the map function
  uint8_t brightness = map(mic_value, 0, MAX_THRESHOLD, 0, 255);

  //Set, but not show, the color and brightness of all the LEDs in the LED Matrix
  for (uint16_t j = 0; j < NUMPIXELS; ++j) {
    matrix.setPixelColor(j, matrix.Color(color_matrix[current_selection][0] * brightness, color_matrix[current_selection][1] * brightness, color_matrix[current_selection][2] * brightness));
  }

  //Actually execute and show the set values on the LEDs
  matrix.show();

  delay(1);
}
